@@ -6,10 +6,9 @@ import os
 import errno
 import sys
 import time
-import traceback
 import functools
 import contextlib
-import warnings
+import logging
 import threading
 import Queue
 
@@ -189,7 +188,7 @@ def comap(function, target):
 			item = yield
 			mappedItem = function(*item)
 			target.send(mappedItem)
-		except StandardError, e:
+		except Exception, e:
 			target.throw(e.__class__, e.message)
 
 
@@ -215,7 +214,7 @@ def queue_sink(queue):
 		try:
 			item = yield
 			queue.put((None, item))
-		except StandardError, e:
+		except Exception, e:
 			queue.put((e.__class__, str(e)))
 		except GeneratorExit:
 			queue.put((GeneratorExit, None))
@@ -394,19 +393,14 @@ class ErrorDisplay(object):
 		else:
 			self.__show_message(message)
 
-	def push_exception_with_lock(self, exception = None, stacklevel=3):
+	def push_exception_with_lock(self):
 		with gtk_lock():
-			self.push_exception(exception, stacklevel=stacklevel)
+			self.push_exception()
 
-	def push_exception(self, exception = None, stacklevel=2):
-		if exception is None:
-			userMessage = str(sys.exc_value)
-			warningMessage = str(traceback.format_exc())
-		else:
-			userMessage = str(exception)
-			warningMessage = str(exception)
+	def push_exception(self):
+		userMessage = str(sys.exc_value)
 		self.push_message(userMessage)
-		warnings.warn(warningMessage, stacklevel=stacklevel)
+		logging.exception(userMessage)
 
 	def pop_message(self):
 		if 0 < len(self.__messages):
@@ -445,11 +439,8 @@ class DummyErrorDisplay(object):
 			self.__show_message(message)
 
 	def push_exception(self, exception = None):
-		if exception is None:
-			warningMessage = traceback.format_exc()
-		else:
-			warningMessage = exception
-		warnings.warn(warningMessage, stacklevel=3)
+		userMessage = str(sys.exc_value)
+		logging.exception(userMessage)
 
 	def pop_message(self):
 		if 0 < len(self.__messages):
@@ -457,7 +448,7 @@ class DummyErrorDisplay(object):
 			del self.__messages[0]
 
 	def __show_message(self, message):
-		warnings.warn(message, stacklevel=2)
+		logging.debug(message)
 
 
 class MessageBox(gtk.MessageDialog):
@@ -530,8 +521,8 @@ class PopupCalendar(object):
 		try:
 			self._calendar.select_month(self._displayDate.month, self._displayDate.year)
 			self._calendar.select_day(self._displayDate.day)
-		except StandardError, e:
-			warnings.warn(e.message)
+		except Exception, e:
+			logging.exception(e.message)
 
 
 class QuickAddView(object):
@@ -581,7 +572,7 @@ class QuickAddView(object):
 			self._taskNameEntry.set_text("")
 
 			self._signalSink.stage.send(("add", name))
-		except StandardError, e:
+		except Exception, e:
 			self._errorDisplay.push_exception()
 
 	def _on_add_edit(self, *args):
@@ -590,13 +581,13 @@ class QuickAddView(object):
 			self._taskNameEntry.set_text("")
 
 			self._signalSink.stage.send(("add-edit", name))
-		except StandardError, e:
+		except Exception, e:
 			self._errorDisplay.push_exception()
 
 	def _on_add_pressed(self, widget):
 		try:
 			self._addToEditTimerId = gobject.timeout_add(1000, self._on_add_edit)
-		except StandardError, e:
+		except Exception, e:
 			self._errorDisplay.push_exception()
 
 	def _on_add_released(self, widget):
@@ -604,7 +595,7 @@ class QuickAddView(object):
 			if self._addToEditTimerId is not None:
 				gobject.source_remove(self._addToEditTimerId)
 			self._addToEditTimerId = None
-		except StandardError, e:
+		except Exception, e:
 			self._errorDisplay.push_exception()
 
 	def _on_paste(self, *args):
@@ -614,13 +605,13 @@ class QuickAddView(object):
 			if addedText:
 				entry += addedText
 			self._taskNameEntry.set_text(entry)
-		except StandardError, e:
+		except Exception, e:
 			self._errorDisplay.push_exception()
 
 	def _on_clear(self, *args):
 		try:
 			self._taskNameEntry.set_text("")
-		except StandardError, e:
+		except Exception, e:
 			self._errorDisplay.push_exception()
 
 
